/*/////////////////////////////////////////////////////////////////////////////
File Name : PPP.C

Author : Rene Trenado

Location : Motorola Applications Lab, Baja California

Date Created : September 2000

Current Revision : 0.0

Notes : This file contains the code for the PPP module 
/////////////////////////////////////////////////////////////////////////////*/
#include <iogp20.h>
#include <string.h>
#include "CommDrv.h"
#include "ppp.h"
#include "IP.h"
#include "Udp.h"
#include "ICMP.h"


const char * User = 	"MyName";		// Username of ISP account
const char * Password = "MyPassword";		// Password of username


/************************* Private Functions *************************/
static void HandleLCPOptions (void);
static void HandleIPCPOptions (void);
static WORD PPPfcs16 (WORD fcs, BYTE *cp, int len);
static void RejectProtocol (BYTE *InBuffer);


////////////////////// Protected ROM Data /////////////////////////
static const BYTE PPPData [] = {
	0xff,0x03,0xc0,0x21,0x02,0x01,0x00,0x04,0x00,0x00
};

static const BYTE LCPTerminate[] = {
	0xff,0x03,0xc0,0x21,0x05,0x04,0x00,0x04,0x80,0xfe
};

static const unsigned short fcstab[256] = {
	0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
	0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
	0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
	0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
	0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
	0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
	0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
	0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
	0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
	0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
	0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
	0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
	0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
	0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
	0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738,
	0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
	0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
	0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
	0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
	0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
	0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
	0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
	0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
	0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
	0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
	0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
	0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
	0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
	0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
	0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
	0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
	0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0x0f78
};

///////////////////// Public  R A M  Data /////////////////////////////
volatile BYTE		PPPStatus = 0;
BYTE			InBuffer [PPP_BUFFER_SIZE + 1];	/// Input Buffer for PPP data
BYTE			OutBuffer[PPP_BUFFER_SIZE + 1];	/// Output Buffer for PPP data


///////////////////// Protected  R A M   Data /////////////////////////
static BYTE		*PPP_Packet = InBuffer;
static volatile BYTE 	FrameSize = 0;
static EventProc 	PPPEntryProc;


/***********************************************************************
Function : 	PPPInit 

Parameters : 	None

Date : 		September 2000

Desc : 		Initialize the PPP Module
***********************************************************************/

void PPPInit (void) {
	PPPStatus |= ReSync;
}



/***********************************************************************
Function : 	PPPGetInputBuffer 

Parameters : 	None

Date : 		September 2000

Desc : 		Returns a PPP Input Buffer pointer to caller

***********************************************************************/

BYTE *PPPGetInputBuffer (void) {
	return &InBuffer[0];
}



/***********************************************************************
Function : 	PPPGetOutputBuffer 

Parameters : 	None

Date : 		September 2000

Desc : 		Returns a pointer to PPP Output Buffer to caller

***********************************************************************/

BYTE *PPPGetOutputBuffer (void) {
	return &OutBuffer[0];
}




/***********************************************************************
Function : 	PPPfcs16 

Parameters : 	fcs: current fcs
		cp: pointer to PPP data
		len: size of PPP data

Date : 		September 2000

Desc : 		Calculate a new fcs given the current fcs and the new data.

***********************************************************************/

static WORD PPPfcs16 (WORD fcs, BYTE *cp, int len) {
	   while (len--) 
		   fcs = (fcs >> 8) ^ fcstab[(fcs ^ *cp++) & 0xff];
       return (fcs);
}


/***********************************************************************
Function : 	public PPPGetChecksum 

Parameters : 	cp:		A pinter to the PPP Packet 
		len:		Size of PPP Packet		

Date : 		September 2000

Desc : 		Returns the Checksum of the PPP Packet pointed by cp
***********************************************************************/

WORD PPPGetChecksum (register unsigned char *cp, register int len) {
	   return ~PPPfcs16( PPPINITFCS16, cp, len );
}



/***********************************************************************
Function : 	ProcPPPReceive 

Parameters : 	A Byte character to stream in a PPP Packet

Date : 		August 2000

Desc : 		This function process a BYTE following HDLC - PPP 
		specifications. The Async event on input driver should 
		call this function (usually the COMM ISR).
***********************************************************************/

void ProcPPPReceive (register BYTE c) {

	PPPStatus |= ByteRx;

	if (PPPStatus & IsFrame) return;

	if (PPPStatus & ReSync) {
		if (c != 0x7E) return;
		PPPStatus &= ~ReSync;
		FrameSize = 0;
	}

	if (PPPStatus & IsESC) {
		PPP_Packet [FrameSize++] = 0x20 ^ c;
		PPPStatus &= ~IsESC;
	}
	else {
		switch (c) {
			case ESC:		//// Special ESC (0x7D) Character received
				PPPStatus |= IsESC;
			break;

			case END:		//// Special END (0x7E) Character received
				if (FrameSize > 0) {	//// Avoid cero length packets (0x7F - 0x7F conditions);
					PPP_Packet [FrameSize] = 0;
					PPPStatus |= IsFrame;	//// Signal Frame availability
				}
			break;

			default:
				PPP_Packet [FrameSize++] = c;
				if (FrameSize > (PPP_BUFFER_SIZE - 6)) {
					FrameSize = 0;
					PPPStatus |= ReSync;
				}
			break;
		}
        }


}


/***********************************************************************
Function : 	PPPSend 

Parameters : 	Buffer:	A pointer to a buffer containing the PPP packet to send
		len:	the size of the PPP packet 

Date : 		September 2000

Desc : 		Sends a BYTE array of len length following HDLC - PPP specifications

***********************************************************************/

void ProcPPPSend (BYTE *Buffer, BYTE len) {
WORD Checksum = 0;

	Checksum = PPPGetChecksum (Buffer, Buffer[7] + 4);
	Buffer [Buffer[7]+4] = Checksum & 0xFF;
	Buffer [Buffer[7]+5] = (Checksum >> 8) & 0xFF;

	WriteComm (0x7E);

	while (len--) {
		if ((signed char)*Buffer < (signed char)0x20) {
			WriteComm (0x7D);
			WriteComm (*Buffer ^ 0x20);
		} else {
			switch (*Buffer) {
				case 0x7E:
					WriteComm (0x7D);
					WriteComm (0x5E);
				break;

				case 0x7D:
					WriteComm (0x7D);
					WriteComm (0x5D);
				break;

				default:
					WriteComm (*Buffer);
				break;
			}
		}
		Buffer++;
	}
	WriteComm (0x7E);
}


/***********************************************************************
Function : 	public PPPFrameSize

Parameters : 	None

Date : 		August 2000

Desc : 		Returns the size of the current available PPP packet
		stored in InBuffer. Caller should call this function
		if needed only when the IsFram flag has been signaled. 
		
***********************************************************************/

BYTE PPPFrameSize (void) {
	return FrameSize;
}




/***********************************************************************
Function : 	protected HandleLCPOptions

Parameters : 	None

Date : 		August 2000

Desc : 		State Machine that implements LCP packet negotiation 
		
***********************************************************************/

static void HandleLCPOptions (void) {
BYTE *dest = OutBuffer;					// A pointer to the options of output buffer
BYTE *ptr = (BYTE *)&InBuffer[8];			// A pointer to the options of input buffer



	switch (InBuffer [4] ) {

		//++++++++++++++++++++++++++++++++++++++++++++++++++++
		case TERMINATE:			//"Server Terminate-Request received"
			Move (InBuffer, OutBuffer, InBuffer[7]+6);
			OutBuffer [4] = TERMINATE_ACK;
			ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);					
			PPPStatus &= ~LinkOn;
		break;

		//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		case REQ:
			//////////////////////////////////////////////////////////////////////
			/////// Server requesting option 2 ///////
			//////////////////////////////////////////////////////////////////////
			if ((InBuffer [8] == 0x02) && (InBuffer [7] <= 0x0A))  {

					if ((InBuffer [10] == 0xFF) &&
						(InBuffer [11] == 0xFF) &&
						(InBuffer [12] == 0xFF) &&
						(InBuffer [13] == 0xFF)) {
					
						InBuffer [4] = ACK;
						ProcPPPSend (InBuffer, InBuffer [7] + 6);
						return;
					}

			} else

			//////////////////////////////////////////////////////////////////////
			/////// Server requesting first options, reject all but 3 ///////
			//////////////////////////////////////////////////////////////////////
			if ((InBuffer [8] != 0x03) && (InBuffer [7] > 9)) {	
			BYTE OptionsSize;
			BYTE Option;
			BYTE Size;

				Move (InBuffer, OutBuffer, 8);				// Move LCP header to output buffer
				OutBuffer [4] = REJ;					// Output will be a reject packet
				dest += 8;						// Offset output pointer to LCP options
				OptionsSize = InBuffer[7] - 4;				// Get size of LCP options in bytes
				while (OptionsSize > 0) {				// Is there options to process?
					Option = *ptr;					// Get option number
					Size = *(ptr + 1);				// Get size of this option

					OptionsSize -= Size;				// Reduce the amount of OptionsSize 

					if (Option == 3) {				// Is this option 3? (authentication protocol)
						ptr += Size;				// Remove this option in output packet
						OutBuffer [7] = OutBuffer [7] - Size;	// Set New Packet size
					}
					else {
						while (Size-- ) {			// Copy this option to the output buffer
							*dest++ = *ptr++;
						}
					}

				}
			} else

			//////////////////////////////////////////////////////////////////////
			/////// Server Request CHAP protocol, We reply with
			/////// a suggestion of the PAP protocol instead
			//////////////////////////////////////////////////////////////////////
			if ((InBuffer [8] == 0x03) && (InBuffer [10] == 0xC2)) {
				InBuffer [4] = NAK;					// NAK CHAP protocol
				InBuffer [10] = 0xC0;					// We suggest PAP instead
				ProcPPPSend (InBuffer, InBuffer[7]+6);			// Send the NAK reply
				return;
			} else

			//////////////////////////////////////////////////////////////////////
			/////// Server Request PAP protocol //////////
			/////// We Acknowledge this reply and then we start negotiating
			/////// the Async-Control-Char..., Here we send both packets!!!
			//////////////////////////////////////////////////////////////////////
				if ((InBuffer [8] == 0x03) && (InBuffer [10] == 0xC0)) {
					Move (InBuffer, OutBuffer, InBuffer[7]+6);
					OutBuffer[4] = ACK;

					ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);

					OutBuffer[4] = REQ;
					OutBuffer[5] = OutBuffer [5] + 1;
					OutBuffer[7] = 0x0A;
					OutBuffer[8] = 0x02;
					OutBuffer[9] = 0x06;
					OutBuffer[10] = 0xFF;
					OutBuffer[11] = 0xFF;
					OutBuffer[12] = 0xFF;
					OutBuffer[13] = 0xFF;

			}
		ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);
		break;

		//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		case ACK:
			//////////////////////////////////////////////////////////////////////
			/////// Server Acknowledge Async Control //////////
			//////////////////////////////////////////////////////////////////////
			if (InBuffer [8] == 0x02) {
				SendPAPPacket (REQ, InBuffer[5] + 1, User, Password);
			}
		break;

		//++++++++++++++++++++++++++++++++++++++++++++++++++++
		case NAK:
		break;

		//++++++++++++++++++++++++++++++++++++++++++++++++++++
		case REJ:
		break;
		
		//++++++++++++++++++++++++++++++++++++++++++++++++++++
		case TERMINATE_ACK:			// Terminate ACK!
			PPPStatus &= ~LinkOn;
		break;

	}
	return;
}



/***********************************************************************
Function : 	protected HandleIPCPOptions

Parameters : 	None

Date : 		August 2000

Desc : 		State Machine that implement IPCP packet negotiation 
		
***********************************************************************/

static void HandleIPCPOptions (void) {
BYTE *dest = (BYTE *)&OutBuffer[8];
BYTE *ptr = (BYTE *)&InBuffer[8];
BYTE FrameSize;
BYTE Option;
BYTE Size;



	switch (InBuffer [4] ) {
		case REQ:
			if ((InBuffer [8] != 0x03) && (InBuffer [7] > 0x0A)) {
					OutBuffer [0] = 0xFF;		// Build a IPCP header 
					OutBuffer [1] = 0x03;
					OutBuffer [2] = 0x80;		// Set IPCP protocol field
					OutBuffer [3] = 0x21;
					OutBuffer [4] = REJ;		// This will be REJ packet for now
					OutBuffer [5] = InBuffer [5];
					FrameSize = InBuffer[7] - 4;
					///////// Ignore all but option #3 ///////
					while (FrameSize > 0) {
						Option = *ptr;
						Size = *(ptr + 1);

						FrameSize -= Size;

						if (Option == 3) {
							ptr += Size;
							OutBuffer [7] = InBuffer [7] - Size;	//Set New Packet size
						}
						else {
							while (Size-- ) {
								*dest++ = *ptr++;
							}
						}
					}
			}
			else {
					//// Acknowledge IP Address ////
					Move (InBuffer, OutBuffer, InBuffer[7]+6);
					OutBuffer [4] = ACK;


					ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);


					//// Now Request IP address to complete 3-way handshake
          				OutBuffer [4] = REQ;			// Request command
					OutBuffer [5] = OutBuffer [5] + 1;	// Packet ID = ID + 1
					OutBuffer [10] = 0;			// IP address is set to 0 so ISP server 
					OutBuffer [11] = 0;			// can assing one to us
					OutBuffer [12] = 0;
					OutBuffer [13] = 0;

			}
			ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);

		break;

		case ACK:
			if (InBuffer [8] == 3) {			// Reply of the only IPCP Request we can send
				IPAddress [0] = InBuffer [10];		// ISP assigned IP address
				IPAddress [1] = InBuffer [11];		
				IPAddress [2] = InBuffer [12];
				IPAddress [3] = InBuffer [13];
				
				PORTC = 0xFF;
				PPPStatus |= LinkOn;			// PPP Link is now up
			}
		break;

		case NAK:
			if ((InBuffer [8] == 0x03) && (InBuffer [7] <= 0x0A)) {
					/// Request IP Address ////
					Move (InBuffer, OutBuffer, InBuffer[7]+6);
					OutBuffer [4] = 0x01;

					ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);
			}
		break;

		case REJ:
		break;
	}

}


/***********************************************************************
Function : 	public PPPSendPAPPacket 

Parameters : 	Action: 	REQ, REJ, NAK
		ID:		Sequence number of PPP packet
		user:		User name for login
		password:	Password in plain text

Date : 		September 2000

Desc : 		Formats a PAP packet on Output Buffer. This function
		supports the type field for future implementation of
		the PPP module in server mode.
***********************************************************************/

void SendPAPPacket (BYTE Action, BYTE ID, char* user, char* password) {
	OutBuffer [0] = 0xFF;
	OutBuffer [1] = 0x03;
	OutBuffer [2] = 0xC0;
	OutBuffer [3] = 0x23;
	OutBuffer [4] = Action;
	OutBuffer [5] = InBuffer [5] + 1;
	OutBuffer [6] = 0;
	OutBuffer [7] = strlen (user) + strlen (password) + 6;
	OutBuffer [8] = strlen (user);
	Move (user, &OutBuffer [9], strlen (user));
	OutBuffer [9 + strlen (user)] = strlen (password);
	Move (password, &OutBuffer [10 + strlen (user)], strlen (password));
	ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);
}

/***********************************************************************
Function : 	Move

Parameters : 	src:	 	A pointer to the data to copy 
		dest:		A pointer to the destination location 
		numBYTEs:	Number of bytes to copy

Date : 		September 2000

Desc : 		Copies a block of numBYTEs bytes from src pointer 
		to dest pointer
***********************************************************************/
void Move (BYTE *src, BYTE *dest, register numBYTEs) {
	if ( numBYTEs <= 0 ) return;
	if ( src < dest ) {
		src += numBYTEs;
		dest += numBYTEs;
		do {
		*--dest = *--src;
		} while ( --numBYTEs > 0 );
	} else
		do {
		     *dest++ = *src++;
		} while ( --numBYTEs > 0 );
}




/***********************************************************************
Function : 	protected RejectProtocol  

Parameters : 	InBuffer -> A pointer to the buffer that has the PPP
		Packet to reject

Date : 		August 2000

Desc : 		Rejects the a PPP packet based on its Protocol field
		Stored on InBuffer
		
***********************************************************************/

static void RejectProtocol (BYTE *InBuffer) {

	OutBuffer [0] = 0xFF;
	OutBuffer [1] = 0x03;
	OutBuffer [2] = 0xC0;
	OutBuffer [3] = 0x21;
	OutBuffer [4] = 0x08;
	OutBuffer [5] = 20;
	OutBuffer [6] = 0;
	OutBuffer [7] = InBuffer[7] + 6;
	Move (&InBuffer[2], &OutBuffer[8], InBuffer [7] + 2);
	ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);
}



/***********************************************************************
Function : 	protected PPPSendVoidLcp 

Parameters : 	None

Date : 		September 2000

Desc : 		Sends a void LCP packet with no options to the PPP Server.
		This will force the server to reply with his options to 
		negotiate. Some ISPs require scripts to stablish a connection thus
		a void LCP packet will try to force the server to negotiate PPP.  
***********************************************************************/

void PPPSendVoidLCP (void) {
WORD Checksum;

	Move (PPPData, OutBuffer, PPPData[7] + 6);
	ProcPPPSend ((BYTE *)OutBuffer, OutBuffer[7] + 6);
}


/***********************************************************************
Function : 	PPPTerminate

Parameters : 	None

Date : 		September 2000

Desc : 		Terminates a PPP link by sending a terminate LCP packet
***********************************************************************/
void PPPTerminate (void) {
   Move ((BYTE *)LCPTerminate, OutBuffer, 10);
   ProcPPPSend (OutBuffer, 10);
}


/***********************************************************************
Function : 	PPPEntry  

Parameters : 	None

Date : 		August 2000

Desc : 		PPP Module Entry, Applications should call PPPEntry 
		frequently in the main loop or in portions of the app 
		code.
***********************************************************************/

void PPPEntry (void) {
		if (PPPStatus & IsFrame) {
		switch (*(WORD *)(&InBuffer [2])) {

				case LCP_PACKET:	
					HandleLCPOptions ();
				break;

				case PAP_PACKET:
					if (InBuffer [4] == 0x02) {	// Authentication OK
						NoOperation;
                    			}
				break;

				case IPCP_PACKET:  			// IPCP Handler
					HandleIPCPOptions ();
				break;

				case IP_DATAGRAM:			// IP Data Handler 
					if (!IPCompare ((BYTE *)&InBuffer [20])) {
									// Misrouted datagram or broadcast message received 
						
					}
					else		
						switch (InBuffer [13]) {

							case UDP:
					                	UDP_Handler ((UDPDatagram *)&InBuffer[16]);							
							break;

							case TCP:
							break;
	
							case ICMP:
								IcmpHandler ((IPDatagram *)&InBuffer[4]);
							break;

							default:
							break;								
						}
				break;


				default:
					RejectProtocol (InBuffer);	// Cannot handle this type of packet 
				break;
			};
    
			PPPStatus &= ~IsFrame;
			PPPStatus |= ReSync;
		}

}
